@@ -58,7 +58,7 @@ foreach ($config as $optname => $optval) {
 }
 
 // framework constants
-define('RCUBE_VERSION', '1.5.1');
+define('RCUBE_VERSION', '1.6.1');
 define('RCUBE_CHARSET', 'UTF-8');
 define('RCUBE_TEMP_FILE_PREFIX', 'RCMTEMP');
 
@@ -113,10 +113,14 @@ PEAR::setErrorHandling(PEAR_ERROR_CALLBACK, function($err) { rcube::raise_error(
  */
 function in_array_nocase($needle, $haystack)
 {
+    if (!is_string($needle) || !is_array($haystack)) {
+        return false;
+    }
+
     // use much faster method for ascii
     if (is_ascii($needle)) {
         foreach ((array) $haystack as $value) {
-            if (strcasecmp($value, $needle) === 0) {
+            if (is_string($value) && strcasecmp($value, $needle) === 0) {
                 return true;
             }
         }
@@ -124,7 +128,7 @@ function in_array_nocase($needle, $haystack)
     else {
         $needle = mb_strtolower($needle);
         foreach ((array) $haystack as $value) {
-            if ($needle === mb_strtolower($value)) {
+            if (is_string($value) && $needle === mb_strtolower($value)) {
                 return true;
             }
         }
@@ -322,7 +326,7 @@ function array_first($array)
 function asciiwords($str, $css_id = false, $replace_with = '')
 {
     $allowed = 'a-z0-9\_\-' . (!$css_id ? '\.' : '');
-    return preg_replace("/[^$allowed]+/i", $replace_with, $str);
+    return preg_replace("/[^$allowed]+/i", $replace_with, (string) $str);
 }
 
 /**
@@ -336,7 +340,7 @@ function asciiwords($str, $css_id = false, $replace_with = '')
 function is_ascii($str, $control_chars = true)
 {
     $regexp = $control_chars ? '/[^\x00-\x7F]/' : '/[^\x20-\x7E]/';
-    return preg_match($regexp, $str) ? false : true;
+    return preg_match($regexp, (string) $str) ? false : true;
 }
 
 /**
