@@ -1,10 +1,11 @@
 <?php
 
-/*
+/**
  +-----------------------------------------------------------------------+
  | This file is part of the Roundcube Webmail client                     |
- | Copyright (C) 2005-2012, The Roundcube Dev Team                       |
- | Copyright (C) 2012, Kolab Systems AG                                  |
+ |                                                                       |
+ | Copyright (C) The Roundcube Dev Team                                  |
+ | Copyright (C) Kolab Systems AG                                        |
  |                                                                       |
  | Licensed under the GNU General Public License version 3 or            |
  | any later version with exceptions for skins & plugins.                |
@@ -23,8 +24,6 @@
  *
  * @package    Framework
  * @subpackage Storage
- * @author     Thomas Bruederli <roundcube@gmail.com>
- * @author     Aleksander Machniak <alec@alec.pl>
  */
 abstract class rcube_storage
 {
@@ -40,14 +39,22 @@ abstract class rcube_storage
      *
      * @var array
      */
-    public static $folder_types = array('drafts', 'sent', 'junk', 'trash');
+    public static $folder_types = ['drafts', 'sent', 'junk', 'trash'];
 
-    protected $folder = 'INBOX';
+    protected $folder          = 'INBOX';
     protected $default_charset = 'ISO-8859-1';
+    protected $options         = ['auth_type' => 'check', 'language' => 'en_US'];
+    protected $page_size       = 10;
+    protected $list_page       = 1;
+    protected $threading       = false;
     protected $search_set;
-    protected $options = array('auth_type' => 'check');
-    protected $page_size = 10;
-    protected $threading = false;
+
+    /**
+     * Internal (in-memory) cache
+     *
+     * @var array
+     */
+    protected $icache = [];
 
     /**
      * All (additional) headers used (in any way) by Roundcube
@@ -56,18 +63,19 @@ abstract class rcube_storage
      *
      * @var array
      */
-    protected $all_headers = array(
-        'IN-REPLY-TO',
-        'BCC',
-        'SENDER',
-        'MESSAGE-ID',
+    protected $all_headers = [
         'CONTENT-TRANSFER-ENCODING',
-        'REFERENCES',
-        'X-DRAFT-INFO',
+        'BCC',
+        'IN-REPLY-TO',
         'MAIL-FOLLOWUP-TO',
         'MAIL-REPLY-TO',
+        'MESSAGE-ID',
+        'REFERENCES',
+        'RESENT-BCC',
         'RETURN-PATH',
-    );
+        'SENDER',
+        'X-DRAFT-INFO',
+    ];
 
     const UNKNOWN       = 0;
     const NOPERM        = 1;
@@ -79,6 +87,8 @@ abstract class rcube_storage
     const NONEXISTENT   = 7;
     const CONTACTADMIN  = 8;
 
+    const DUAL_USE_FOLDERS = 'X-DUAL-USE-FOLDERS';
+
 
     /**
      * Connect to the server
@@ -89,25 +99,22 @@ abstract class rcube_storage
      * @param  integer  $port    Port to connect to
      * @param  string   $use_ssl SSL schema (either ssl or tls) or null if plain connection
      *
-     * @return boolean  TRUE on success, FALSE on failure
+     * @return bool True on success, False on failure
      */
     abstract function connect($host, $user, $pass, $port = 143, $use_ssl = null);
 
-
     /**
      * Close connection. Usually done on script shutdown
      */
     abstract function close();
 
-
     /**
      * Checks connection state.
      *
-     * @return boolean  TRUE on success, FALSE on failure
+     * @return bool True on success, False on failure
      */
     abstract function is_connected();
 
-
     /**
      * Check connection state, connect if not connected.
      *
@@ -115,7 +122,6 @@ abstract class rcube_storage
      */
     abstract function check_connection();
 
-
     /**
      * Returns code of last error
      *
@@ -123,7 +129,6 @@ abstract class rcube_storage
      */
     abstract function get_error_code();
 
-
     /**
      * Returns message of last error
      *
@@ -131,7 +136,6 @@ abstract class rcube_storage
      */
     abstract function get_error_str();
 
-
     /**
      * Returns code of last command response
      *
@@ -139,7 +143,6 @@ abstract class rcube_storage
      */
     abstract function get_response_code();
 
-
     /**
      * Set connection and class options
      *
@@ -147,10 +150,9 @@ abstract class rcube_storage
      */
     public function set_options($opt)
     {
-        $this->options = array_merge($this->options, (array)$opt);
+        $this->options = array_merge($this->options, (array) $opt);
     }
 
-
     /**
      * Get connection/class option
      *
@@ -163,33 +165,30 @@ abstract class rcube_storage
         return $this->options[$name];
     }
 
-
     /**
      * Activate/deactivate debug mode.
      *
-     * @param boolean $dbg True if conversation with the server should be logged
+     * @param bool $dbg True if conversation with the server should be logged
      */
     abstract function set_debug($dbg = true);
 
-
     /**
      * Set default message charset.
      *
      * This will be used for message decoding if a charset specification is not available
      *
-     * @param  string $cs Charset string
+     * @param string $cs Charset string
      */
     public function set_charset($cs)
     {
         $this->default_charset = $cs;
     }
 
-
     /**
      * Set internal folder reference.
-     * All operations will be perfomed on this folder.
+     * All operations will be performed on this folder.
      *
-     * @param  string $folder  Folder name
+     * @param string $folder Folder name
      */
     public function set_folder($folder)
     {
@@ -200,7 +199,6 @@ abstract class rcube_storage
         $this->folder = $folder;
     }
 
-
     /**
      * Returns the currently used folder name
      *
@@ -211,7 +209,6 @@ abstract class rcube_storage
         return $this->folder;
     }
 
-
     /**
      * Set internal list page number.
      *
@@ -219,10 +216,11 @@ abstract class rcube_storage
      */
     public function set_page($page)
     {
-        $this->list_page = (int) $page;
+        if ($page = intval($page)) {
+            $this->list_page = $page;
+        }
     }
 
-
     /**
      * Gets internal list page number.
      *
@@ -233,7 +231,6 @@ abstract class rcube_storage
         return $this->list_page;
     }
 
-
     /**
      * Set internal page size
      *
@@ -244,7 +241,6 @@ abstract class rcube_storage
         $this->page_size = (int) $size;
     }
 
-
     /**
      * Get internal page size
      *
@@ -255,15 +251,13 @@ abstract class rcube_storage
         return $this->page_size;
     }
 
-
     /**
      * Save a search result for future message listing methods.
      *
-     * @param  mixed  $set  Search set in driver specific format
+     * @param mixed $set Search set in driver specific format
      */
     abstract function set_search_set($set);
 
-
     /**
      * Return the saved search set.
      *
@@ -271,95 +265,87 @@ abstract class rcube_storage
      */
     abstract function get_search_set();
 
-
     /**
      * Returns the storage server's (IMAP) capability
      *
-     * @param   string  $cap Capability name
+     * @param string $cap Capability name
      *
-     * @return  mixed   Capability value or TRUE if supported, FALSE if not
+     * @return mixed Capability value or True if supported, False if not
      */
     abstract function get_capability($cap);
 
-
     /**
      * Sets threading flag to the best supported THREAD algorithm.
      * Enable/Disable threaded mode.
      *
-     * @param  boolean  $enable TRUE to enable and FALSE
+     * @param bool $enable True to enable threading
      *
-     * @return mixed   Threading algorithm or False if THREAD is not supported
+     * @return mixed Threading algorithm or False if THREAD is not supported
      */
     public function set_threading($enable = false)
     {
         $this->threading = false;
 
         if ($enable && ($caps = $this->get_capability('THREAD'))) {
-            $methods = array('REFS', 'REFERENCES', 'ORDEREDSUBJECT');
+            $methods = ['REFS', 'REFERENCES', 'ORDEREDSUBJECT'];
             $methods = array_intersect($methods, $caps);
 
-            $this->threading = array_shift($methods);
+            $this->threading = array_first($methods);
         }
 
         return $this->threading;
     }
 
-
     /**
      * Get current threading flag.
      *
-     * @return mixed  Threading algorithm or False if THREAD is not supported or disabled
+     * @return mixed Threading algorithm or False if THREAD is not supported or disabled
      */
     public function get_threading()
     {
         return $this->threading;
     }
 
-
     /**
      * Checks the PERMANENTFLAGS capability of the current folder
      * and returns true if the given flag is supported by the server.
      *
-     * @param   string  $flag Permanentflag name
+     * @param string $flag Permanentflag name
      *
-     * @return  boolean True if this flag is supported
+     * @return bool True if this flag is supported
      */
     abstract function check_permflag($flag);
 
-
     /**
      * Returns the delimiter that is used by the server
      * for folder hierarchy separation.
      *
-     * @return  string  Delimiter string
+     * @return string Delimiter string
      */
     abstract function get_hierarchy_delimiter();
 
-
     /**
      * Get namespace
      *
      * @param string $name Namespace array index: personal, other, shared, prefix
      *
-     * @return  array  Namespace data
+     * @return array Namespace data
      */
     abstract function get_namespace($name = null);
 
-
     /**
      * Get messages count for a specific folder.
      *
-     * @param  string  $folder  Folder name
-     * @param  string  $mode    Mode for count [ALL|THREADS|UNSEEN|RECENT|EXISTS]
-     * @param  boolean $force   Force reading from server and update cache
-     * @param  boolean $status  Enables storing folder status info (max UID/count),
-     *                          required for folder_status()
+     * @param string $folder  Folder name
+     * @param string $mode    Mode for count [ALL|THREADS|UNSEEN|RECENT|EXISTS]
+     * @param bool   $force   Force reading from server and update cache
+     * @param bool   $status  Enables storing folder status info (max UID/count),
+     *                        required for folder_status()
      *
-     * @return int     Number of messages
+     * @return int Number of messages
      */
     abstract function count($folder = null, $mode = 'ALL', $force = false, $status = true);
 
-
     /**
      * Public method for listing message flags
      *
@@ -371,7 +357,6 @@ abstract class rcube_storage
      */
     abstract function list_flags($folder, $uids, $mod_seq = null);
 
-
     /**
      * Public method for listing headers.
      *
@@ -385,7 +370,6 @@ abstract class rcube_storage
      */
     abstract function list_messages($folder = null, $page = null, $sort_field = null, $sort_order = null, $slice = 0);
 
-
     /**
      * Return sorted list of message UIDs
      *
@@ -397,7 +381,6 @@ abstract class rcube_storage
      */
     abstract function index($folder = null, $sort_field = null, $sort_order = null);
 
-
     /**
      * Invoke search request to the server.
      *
@@ -406,11 +389,10 @@ abstract class rcube_storage
      * @param  string  $charset    Search charset
      * @param  string  $sort_field Header field to sort by
      *
-     * @todo: Search criteria should be provided in non-IMAP format, eg. array
+     * @todo: Search criteria should be provided in non-IMAP format, e.g. array
      */
     abstract function search($folder = null, $str = 'ALL', $charset = null, $sort_field = null);
 
-
     /**
      * Direct (real and simple) search request (without result sorting and caching).
      *
@@ -421,7 +403,6 @@ abstract class rcube_storage
      */
     abstract function search_once($folder = null, $str = 'ALL');
 
-
     /**
      * Refresh saved search set
      *
@@ -445,7 +426,6 @@ abstract class rcube_storage
      */
     abstract function get_message($uid, $folder = null);
 
-
     /**
      * Return message headers object of a specific message
      *
@@ -457,26 +437,24 @@ abstract class rcube_storage
      */
     abstract function get_message_headers($uid, $folder = null, $force = false);
 
-
     /**
      * Fetch message body of a specific message from the server
      *
      * @param  int                $uid    Message UID
      * @param  string             $part   Part number
      * @param  rcube_message_part $o_part Part object created by get_structure()
-     * @param  mixed              $print  True to print part, ressource to write part contents in
+     * @param  mixed              $print  True to print part, resource to write part contents in
      * @param  resource           $fp     File pointer to save the message part
-     * @param  boolean            $skip_charset_conv Disables charset conversion
+     * @param  bool               $skip_charset_conv Disables charset conversion
      *
      * @return string Message/part body if not printed
      */
     abstract function get_message_part($uid, $part = 1, $o_part = null, $print = null, $fp = null, $skip_charset_conv = false);
 
-
     /**
      * Fetch message body of a specific message from the server
      *
-     * @param  int    $uid  Message UID
+     * @param int $uid Message UID
      *
      * @return string $part Message/part body
      * @see    rcube_imap::get_message_part()
@@ -485,30 +463,29 @@ abstract class rcube_storage
     {
         $headers = $this->get_message_headers($uid);
         return rcube_charset::convert($this->get_message_part($uid, $part, null),
-            $headers->charset ? $headers->charset : $this->default_charset);
+            $headers->charset ?: $this->default_charset);
     }
 
-
     /**
      * Returns the whole message source as string (or saves to a file)
      *
-     * @param int      $uid Message UID
-     * @param resource $fp  File pointer to save the message
+     * @param int      $uid  Message UID
+     * @param resource $fp   File pointer to save the message
+     * @param string   $part Optional message part ID
      *
      * @return string Message source string
      */
-    abstract function get_raw_body($uid, $fp = null);
-
+    abstract function get_raw_body($uid, $fp = null, $part = null);
 
     /**
      * Returns the message headers as string
      *
-     * @param int $uid  Message UID
+     * @param int    $uid  Message UID
+     * @param string $part Optional message part ID
      *
      * @return string Message headers string
      */
-    abstract function get_raw_headers($uid);
-
+    abstract function get_raw_headers($uid, $part = null);
 
     /**
      * Sends the whole message source to stdout
@@ -518,20 +495,18 @@ abstract class rcube_storage
      */
     abstract function print_raw_body($uid, $formatted = true);
 
-
     /**
      * Set message flag to one or several messages
      *
-     * @param mixed   $uids       Message UIDs as array or comma-separated string, or '*'
-     * @param string  $flag       Flag to set: SEEN, UNDELETED, DELETED, RECENT, ANSWERED, DRAFT, MDNSENT
-     * @param string  $folder     Folder name
-     * @param boolean $skip_cache True to skip message cache clean up
+     * @param mixed  $uids       Message UIDs as array or comma-separated string, or '*'
+     * @param string $flag       Flag to set: SEEN, UNDELETED, DELETED, RECENT, ANSWERED, DRAFT, MDNSENT
+     * @param string $folder     Folder name
+     * @param bool   $skip_cache True to skip message cache clean up
      *
-     * @return bool  Operation status
+     * @return bool Operation status
      */
     abstract function set_flag($uids, $flag, $folder = null, $skip_cache = false);
 
-
     /**
      * Remove message flag for one or several messages
      *
@@ -539,7 +514,7 @@ abstract class rcube_storage
      * @param string $flag    Flag to unset: SEEN, DELETED, RECENT, ANSWERED, DRAFT, MDNSENT
      * @param string $folder  Folder name
      *
-     * @return bool   Operation status
+     * @return bool Operation status
      * @see set_flag
      */
     public function unset_flag($uids, $flag, $folder = null)
@@ -547,7 +522,6 @@ abstract class rcube_storage
         return $this->set_flag($uids, 'UN'.$flag, $folder);
     }
 
-
     /**
      * Append a mail message (source) to a specific folder.
      *
@@ -555,14 +529,13 @@ abstract class rcube_storage
      * @param string|array $message The message source string or filename
      *                              or array (of strings and file pointers)
      * @param string       $headers Headers string if $message contains only the body
-     * @param boolean      $is_file True if $message is a filename
+     * @param bool         $is_file True if $message is a filename
      * @param array        $flags   Message flags
      * @param mixed        $date    Message internal date
      *
      * @return int|bool Appended message UID or True on success, False on error
      */
-    abstract function save_message($folder, &$message, $headers = '', $is_file = false, $flags = array(), $date = null);
-
+    abstract function save_message($folder, &$message, $headers = '', $is_file = false, $flags = [], $date = null);
 
     /**
      * Move message(s) from one folder to another.
@@ -571,11 +544,10 @@ abstract class rcube_storage
      * @param string $to    Target folder
      * @param string $from  Source folder
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     abstract function move_message($uids, $to, $from = null);
 
-
     /**
      * Copy message(s) from one mailbox to another.
      *
@@ -583,59 +555,62 @@ abstract class rcube_storage
      * @param string $to    Target folder
      * @param string $from  Source folder
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     abstract function copy_message($uids, $to, $from = null);
 
-
     /**
      * Mark message(s) as deleted and expunge.
      *
      * @param mixed  $uids    Message UIDs as array or comma-separated string, or '*'
      * @param string $folder  Source folder
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     abstract function delete_message($uids, $folder = null);
 
-
     /**
      * Expunge message(s) and clear the cache.
      *
      * @param mixed   $uids        Message UIDs as array or comma-separated string, or '*'
      * @param string  $folder      Folder name
-     * @param boolean $clear_cache False if cache should not be cleared
+     * @param bool    $clear_cache False if cache should not be cleared
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     abstract function expunge_message($uids, $folder = null, $clear_cache = true);
 
-
     /**
      * Parse message UIDs input
      *
-     * @param mixed $uids UIDs array or comma-separated list or '*' or '1:*'
+     * @param mixed $uids Message UIDs as array or comma-separated string, or '*'
+     *                    or rcube_result_index object
      *
      * @return array Two elements array with UIDs converted to list and ALL flag
      */
     protected function parse_uids($uids)
     {
-        if ($uids === '*' || $uids === '1:*') {
+        $all = false;
+
+        if ($uids instanceof rcube_result_index) {
+            $uids = $uids->get_compressed();
+        }
+        else if ($uids === '*' || $uids === '1:*') {
             if (empty($this->search_set)) {
                 $uids = '1:*';
                 $all = true;
             }
             // get UIDs from current search set
             else {
-                $uids = join(',', $this->search_set->get());
+                $uids = implode(',', $this->search_set->get());
             }
         }
         else {
             if (is_array($uids)) {
-                $uids = join(',', $uids);
+                $uids = implode(',', $uids);
             }
             else if (strpos($uids, ':')) {
-                $uids = join(',', rcube_imap_generic::uncompressMessageSet($uids));
+                $uids = implode(',', rcube_imap_generic::uncompressMessageSet($uids));
             }
 
             if (preg_match('/[^0-9,]/', $uids)) {
@@ -643,12 +618,12 @@ abstract class rcube_storage
             }
         }
 
-        return array($uids, (bool) $all);
+        return [$uids, $all];
     }
 
 
     /* --------------------------------
-     *        folder managment
+     *        folder management
      * --------------------------------*/
 
     /**
@@ -664,7 +639,6 @@ abstract class rcube_storage
      */
     abstract function list_folders_subscribed($root = '', $name = '*', $filter = null, $rights = null, $skip_sort = false);
 
-
     /**
      * Get a list of all folders available on the server.
      *
@@ -678,37 +652,37 @@ abstract class rcube_storage
      */
     abstract function list_folders($root = '', $name = '*', $filter = null, $rights = null, $skip_sort = false);
 
-
     /**
      * Subscribe to a specific folder(s)
      *
      * @param array $folders Folder name(s)
      *
-     * @return boolean True on success
+     * @return bool True on success
      */
     abstract function subscribe($folders);
 
-
     /**
      * Unsubscribe folder(s)
      *
      * @param array $folders Folder name(s)
      *
-     * @return boolean True on success
+     * @return bool True on success
      */
     abstract function unsubscribe($folders);
 
-
     /**
      * Create a new folder on the server.
      *
      * @param string  $folder    New folder name
-     * @param boolean $subscribe True if the newvfolder should be subscribed
+     * @param bool    $subscribe True if the new folder should be subscribed
+     * @param string  $type      Optional folder type (junk, trash, drafts, sent, archive)
+     * @param bool    $noselect  Make the folder \NoSelect folder by adding hierarchy
+     *                           separator at the end (useful for server that do not support
+     *                           both folders and messages as folder children)
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
-    abstract function create_folder($folder, $subscribe = false);
-
+    abstract function create_folder($folder, $subscribe = false, $type = null, $noselect = false);
 
     /**
      * Set a new name to an existing folder
@@ -716,59 +690,54 @@ abstract class rcube_storage
      * @param string $folder   Folder to rename
      * @param string $new_name New folder name
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     abstract function rename_folder($folder, $new_name);
 
-
     /**
      * Remove a folder from the server.
      *
      * @param string $folder Folder name
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     abstract function delete_folder($folder);
 
-
     /**
      * Send expunge command and clear the cache.
      *
      * @param string  $folder      Folder name
-     * @param boolean $clear_cache False if cache should not be cleared
+     * @param bool    $clear_cache False if cache should not be cleared
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     public function expunge_folder($folder = null, $clear_cache = true)
     {
         return $this->expunge_message('*', $folder, $clear_cache);
     }
 
-
     /**
-     * Remove all messages in a folder..
+     * Remove all messages in a folder.
      *
      * @param string  $folder  Folder name
      *
-     * @return boolean True on success, False on error
+     * @return bool True on success, False on error
      */
     public function clear_folder($folder = null)
     {
         return $this->delete_message('*', $folder);
     }
 
-
     /**
      * Checks if folder exists and is subscribed
      *
-     * @param string   $folder       Folder name
-     * @param boolean  $subscription Enable subscription checking
+     * @param string  $folder       Folder name
+     * @param bool    $subscription Enable subscription checking
      *
-     * @return boolean True if folder exists, False otherwise
+     * @return bool True if folder exists, False otherwise
      */
     abstract function folder_exists($folder, $subscription = false);
 
-
     /**
      * Get folder size (size of all messages in a folder)
      *
@@ -778,7 +747,6 @@ abstract class rcube_storage
      */
     abstract function folder_size($folder);
 
-
     /**
      * Returns the namespace where the folder is in
      *
@@ -788,7 +756,6 @@ abstract class rcube_storage
      */
     abstract function folder_namespace($folder);
 
-
     /**
      * Gets folder attributes (from LIST response, e.g. \Noselect, \Noinferiors).
      *
@@ -799,7 +766,6 @@ abstract class rcube_storage
      */
     abstract function folder_attributes($folder, $force = false);
 
-
     /**
      * Gets connection (and current folder) data: UIDVALIDITY, EXISTS, RECENT,
      * PERMANENTFLAGS, UIDNEXT, UNSEEN
@@ -810,7 +776,6 @@ abstract class rcube_storage
      */
     abstract function folder_data($folder);
 
-
     /**
      * Returns extended information about the folder.
      *
@@ -820,7 +785,6 @@ abstract class rcube_storage
      */
     abstract function folder_info($folder);
 
-
     /**
      * Returns current status of a folder (compared to the last time use)
      *
@@ -829,8 +793,7 @@ abstract class rcube_storage
      *
      * @return int Folder status
      */
-    abstract function folder_status($folder = null, &$diff = array());
-
+    abstract function folder_status($folder = null, &$diff = []);
 
     /**
      * Synchronizes messages cache.
@@ -839,7 +802,6 @@ abstract class rcube_storage
      */
     abstract function folder_sync($folder);
 
-
     /**
      * Modify folder name according to namespace.
      * For output it removes prefix of the personal namespace if it's possible.
@@ -853,6 +815,25 @@ abstract class rcube_storage
      */
     abstract function mod_folder($folder, $mode = 'out');
 
+    /**
+     * Check if the folder name is valid
+     *
+     * @param string $folder Folder name (UTF-8)
+     * @param string &$char  First forbidden character found
+     *
+     * @return bool True if the name is valid, False otherwise
+     */
+    public function folder_validate($folder, &$char = null)
+    {
+        $delim = $this->get_hierarchy_delimiter();
+
+        if (strpos($folder, $delim) !== false) {
+            $char = $delim;
+            return false;
+        }
+
+        return true;
+    }
 
     /**
      * Create all folders specified as default
@@ -874,7 +855,6 @@ abstract class rcube_storage
         }
     }
 
-
     /**
      * Check if specified folder is a special folder
      */
@@ -883,7 +863,6 @@ abstract class rcube_storage
         return $name == 'INBOX' || in_array($name, $this->get_special_folders());
     }
 
-
     /**
      * Return configured special folders
      */
@@ -892,7 +871,7 @@ abstract class rcube_storage
         // getting config might be expensive, store special folders in memory
         if (!isset($this->icache['special-folders'])) {
             $rcube = rcube::get_instance();
-            $this->icache['special-folders'] = array();
+            $this->icache['special-folders'] = [];
 
             foreach (self::$folder_types as $type) {
                 if ($folder = $rcube->config->get($type . '_mbox')) {
@@ -904,17 +883,15 @@ abstract class rcube_storage
         return $this->icache['special-folders'];
     }
 
-
     /**
      * Set special folder associations stored in backend
      */
     public function set_special_folders($specials)
     {
-        // should be overriden by storage class if backend supports special folders (SPECIAL-USE)
+        // should be overridden by storage class if backend supports special folders (SPECIAL-USE)
         unset($this->icache['special-folders']);
     }
 
-
     /**
      * Get mailbox quota information.
      *
@@ -936,11 +913,10 @@ abstract class rcube_storage
      * @param string $user    User name
      * @param string $acl     ACL string
      *
-     * @return boolean True on success, False on failure
+     * @return bool True on success, False on failure
      */
     abstract function set_acl($folder, $user, $acl);
 
-
     /**
      * Removes any <identifier,rights> pair for the
      * specified user from the ACL for the specified
@@ -949,11 +925,10 @@ abstract class rcube_storage
      * @param string $folder  Folder name
      * @param string $user    User name
      *
-     * @return boolean True on success, False on failure
+     * @return bool True on success, False on failure
      */
     abstract function delete_acl($folder, $user);
 
-
     /**
      * Returns the access control list for a folder (GETACL).
      *
@@ -963,7 +938,6 @@ abstract class rcube_storage
      */
     abstract function get_acl($folder);
 
-
     /**
      * Returns information about what rights can be granted to the
      * user (identifier) in the ACL for the folder (LISTRIGHTS).
@@ -975,7 +949,6 @@ abstract class rcube_storage
      */
     abstract function list_rights($folder, $user);
 
-
     /**
      * Returns the set of rights that the current user has to a folder (MYRIGHTS).
      *
@@ -985,40 +958,37 @@ abstract class rcube_storage
      */
     abstract function my_rights($folder);
 
-
     /**
      * Sets metadata/annotations (SETMETADATA/SETANNOTATION)
      *
      * @param string $folder  Folder name (empty for server metadata)
      * @param array  $entries Entry-value array (use NULL value as NIL)
      *
-     * @return boolean True on success, False on failure
+     * @return bool True on success, False on failure
      */
     abstract function set_metadata($folder, $entries);
 
-
     /**
      * Unsets metadata/annotations (SETMETADATA/SETANNOTATION)
      *
      * @param string $folder  Folder name (empty for server metadata)
      * @param array  $entries Entry names array
      *
-     * @return boolean True on success, False on failure
+     * @return bool True on success, False on failure
      */
     abstract function delete_metadata($folder, $entries);
 
-
     /**
      * Returns folder metadata/annotations (GETMETADATA/GETANNOTATION).
      *
      * @param string $folder   Folder name (empty for server metadata)
      * @param array  $entries  Entries
      * @param array  $options  Command options (with MAXSIZE and DEPTH keys)
+     * @param bool   $force    Disables cache use
      *
      * @return array Metadata entry-value hash array on success, NULL on error
      */
-    abstract function get_metadata($folder, $entries, $options = array());
-
+    abstract function get_metadata($folder, $entries, $options = [], $force = false);
 
     /* -----------------------------------------
      *   Cache related functions
@@ -1028,12 +998,11 @@ abstract class rcube_storage
      * Clears the cache.
      *
      * @param string  $key         Cache key name or pattern
-     * @param boolean $prefix_mode Enable it to clear all keys starting
+     * @param bool    $prefix_mode Enable it to clear all keys starting
      *                             with prefix specified in $key
      */
     abstract function clear_cache($key = null, $prefix_mode = false);
 
-
     /**
      * Returns cached value
      *
@@ -1043,10 +1012,8 @@ abstract class rcube_storage
      */
     abstract function get_cache($key);
 
-
     /**
      * Delete outdated cache entries
      */
     abstract function cache_gc();
-
-}  // end class rcube_storage
+}
